package geocode;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import log.SystemLogger;
import setting.GeocodeSettings;

public class DBManager {

	private String dbURL; // f[^x[X̐ڑ

	private String dbUserName; // f[^x[X̃[U

	private String dbPass; // f[^x[X̃pX[h

	private GeocodeSettings settings; // ݒt@C̏

	private long requestExpiration; // NGXg̗L

	private long dataExpiration; // n̗L

	private int returnMax; // ԂnXg̒

	private Connection con;

	private Statement stat;

	/**
	 * RXgN^
	 */
	public DBManager(GeocodeSettings settings) {
		this.settings = settings;
		initSettings();
		openConnection();
	}

	/**
	 * ݒ
	 */
	void initSettings() {
		dbURL = settings.getDBURL();
		dbUserName = settings.getDBUserName();
		dbPass = settings.getDBPass();
		requestExpiration = settings.getSearchKeyExpiration() * 1000L * 60L
				* 60L * 24L;
		dataExpiration = settings.getCacheExpiration() * 1000L * 60L * 60L
				* 24L;
		returnMax = settings.getReturnMax();
		if (dbPass == null) {
			dbPass = "";
		}
	}

	/**
	 * f[^x[Xɐڑ
	 * 
	 * @return trueAsfalseԂ
	 */
	private boolean openConnection() {
		try {
			Class.forName("org.gjt.mm.mysql.Driver");
			con = DriverManager.getConnection(dbURL, dbUserName, dbPass);
			stat = con.createStatement();
			return true;
		} catch (ClassNotFoundException e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return false;
	}

	/**
	 * f[^x[X̐ڑ
	 * 
	 * @return trueAsfalseԂ
	 */
	public boolean closeConnection() {
		try {
			stat.close();
			con.close();
			return true;
		} catch (Exception e) {
			SystemLogger.writer.writeError(
					SystemLogger.ERROR_DB_DISCONNECT_FAIL, e);
		}
		return false;
	}

	public int getRequestTimes(String searchKey) {
		int requestTimes = 0;
		try {
			// T[`L[̃NGXg񐔂𑝂₷
			String sql = "SELECT requestTimes FROM geocodingrequest "
					+ "WHERE searchQuery = '" + searchKey + "'";
			ResultSet rs = stat.executeQuery(sql);
			while (rs.next())
				requestTimes = rs.getInt("requestTimes");

		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return requestTimes;
	}

	/**
	 * L[f[^x[Xɑ݂邩ǂm߂
	 * 
	 * @param key
	 *            L[
	 * @return L[ɑΉ郌R[hf[^x[Xɂ邩ǂׂB
	 */
	public boolean keyExist(String key) {
		try {
			String sql = "SELECT * FROM geocodingrequest "
					+ "WHERE searchQuery = '" + key + "'";
			ResultSet rs = stat.executeQuery(sql);
			while (rs.next())
				return true;
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return false;
	}

	/**
	 * T[`L[̗Lm߂
	 * 
	 * @param key
	 *            T[`L[
	 * @return ؂̏ꍇtrueȀꍇfalseԂ
	 */
	public boolean keyExpired(String key) {
		try {
			String sql = "SELECT lastUpdatedRequest FROM geocodingrequest "
					+ "WHERE searchQuery = '" + key + "'";
			ResultSet rs = stat.executeQuery(sql);
			while (rs.next()) {
				// ݂̓t擾
				Date dateNow = new Date();
				// ŏIXV擾
				Date date = rs.getDate("lastUpdatedRequest");
				long timeDiff = dateNow.getTime() - date.getTime();
				// L߂Ă邩ǂm߂
				if (timeDiff > requestExpiration)
					return true; // L߂Ă
			}
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return false; // L߂ĂȂ
	}

	/**
	 * T[`L[f[^x[Xɓo^
	 * 
	 * @param searchKey
	 *            T[`L[
	 * @param keyList
	 *            nXg
	 */
	public void addKey(String searchKey, List<String> keyList) {
		try {
			SystemLogger.writer.writeDebug("Adding key to RT :" + searchKey);// fobOvg
			java.sql.Date date = new java.sql.Date((new Date().getTime()));
			String sql = "insert into geocodingrequest " + "values ('"
					+ searchKey + "',1,'" + date + "')";
			stat.executeUpdate(sql);
			int i = 0;
			int rank = 1;
			if (keyList != null) {
				while (i < keyList.size()) {
					String locationName = keyList.get(i);
					SystemLogger.writer.writeDebug("Adding key to LT :"
							+ searchKey + "," + locationName);// fobOvg
					sql = "insert into geocodinglink values ('" + searchKey
							+ "','" + locationName + "'," + rank + ")";
					stat.executeUpdate(sql);
					i++;
					rank++;
				}
			}
		} catch (SQLException e) { // L[̏d˓ꍇ
			SystemLogger.writer
					.writeWarning(SystemLogger.WARNING_DB_INSERT_FAIL);
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
	}

	/**
	 * NGXg̉񐔂XV
	 * 
	 * @param searchKey
	 *            T[`L[
	 */
	public void updateRequestTimes(String searchKey) {
		try {
			// T[`L[̃NGXg񐔂𑝂₷
			String sql = "SELECT requestTimes FROM geocodingrequest "
					+ "WHERE searchQuery = '" + searchKey + "'";
			ResultSet rs = stat.executeQuery(sql);
			int requestTimes = 0;
			while (rs.next())
				requestTimes = rs.getInt("requestTimes");
			if (requestTimes < Integer.MAX_VALUE)
				requestTimes++;
			sql = "update geocodingrequest set requestTimes = " + requestTimes
					+ " where searchQuery = '" + searchKey + "'";
			stat.executeUpdate(sql);
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
	}

	/**
	 * T[`L[̏XV
	 * 
	 * @param searchKey
	 *            T[`L[
	 * @param keyList
	 *            nXg
	 */
	public void updateKey(String searchKey, List<String> keyList) {
		try {
			// T[`L[̗LmF
			if (keyExpired(searchKey)) { // L؂Ăꍇ
				SystemLogger.writer.writeDebug("Updating key to LT :"
						+ searchKey + "," + searchKey);// fobOvg
				// LXV
				java.sql.Date date = new java.sql.Date((new Date().getTime()));
				String sql = "update geocodingrequest set lastUpdatedRequest = '"
						+ date + "' where searchQuery = '" + searchKey + "'";
				stat.executeUpdate(sql);

				// T[`L[ɊYnXgXV
				sql = "DELETE FROM geocodinglink WHERE searchQuery = '"
						+ searchKey + "'";
				stat.executeUpdate(sql);
				int i = 0;
				if (keyList != null) {
					while (i < keyList.size()) {
						String locationName = keyList.get(i);
						SystemLogger.writer.writeDebug("Adding key to LT :"
								+ searchKey + "," + locationName);// fobOvg
						int rank = i + 1;
						sql = "insert into geocodinglink " + "values ('"
								+ searchKey + "','" + locationName + "',"
								+ rank + ")";
						stat.executeUpdate(sql);
						i++;
					}
				}
			}
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					null);
			terminate();
		}
	}

	/**
	 * nɊYn񂪃f[^x[Xɑ݂邩ǂm߂
	 * 
	 * @param key
	 *            n
	 * @return ꍇtrueAȂꍇfalseԂ
	 */
	public boolean dataExist(String key) {
		try {
			String sql = "SELECT * FROM geocodingdata "
					+ "WHERE locationName = '" + key + "'";
			ResultSet rs = stat.executeQuery(sql);
			while (rs.next())
				return true;
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return false;
	}

	/**
	 * n̗Lm߂
	 * 
	 * @param key
	 *            n
	 * @return ؂̏ꍇtrueȀꍇfalseԂ
	 */
	public boolean dataExpired(String key) {
		try {
			String sql = "SELECT lastUpdated FROM geocodingdata "
					+ "WHERE locationName = '" + key + "'";
			ResultSet rs = stat.executeQuery(sql);
			while (rs.next()) {
				// ݂̓t擾
				Date dateNow = new Date();
				// ŏIXV擾
				Date date = rs.getDate("lastUpdated");
				long timeDiff = dateNow.getTime() - date.getTime();
				// L߂Ă邩ǂm߂
				if (timeDiff > dataExpiration)
					return true; // L߂Ă
			}
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return false; // L߂ĂȂ
	}

	/**
	 * f[^x[Xɒnǉ
	 * 
	 * @param data
	 *            n
	 */
	public void addData(GeoData data) {
		try {

			SystemLogger.writer.writeDebug("Adding data to DT :"
					+ data.getLocationName() + "," + data.getLatitude() + ","
					+ data.getLongitude());// fobOvg
			java.sql.Date date = new java.sql.Date(new Date().getTime());
			String sql = "insert into geocodingdata (locationName,latitude,longitude,insertDate,lastUpdated) "
					+ "values('"
					+ data.getLocationName()
					+ "','"
					+ data.getLatitude()
					+ "','"
					+ data.getLongitude()
					+ "','"
					+ date + "','" + date + "')";
			stat.executeUpdate(sql);
		} catch (SQLException e) { // L[̏d˓ꍇ
			SystemLogger.writer
					.writeWarning(SystemLogger.WARNING_DB_INSERT_FAIL);
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
	}

	/**
	 * nXV
	 * 
	 * @param data
	 *            n
	 */
	public void updateData(GeoData data) {
		try {
			SystemLogger.writer.writeDebug("Updating data to DT :"
					+ data.getLocationName() + "," + data.getLatitude() + ","
					+ data.getLongitude());// fobOvg
			java.sql.Date date = new java.sql.Date(new Date().getTime());
			String sql = "update geocodingdata set latitude = "
					+ data.getLatitude() + ", longitude = "
					+ data.getLongitude() + ", lastUpdated = '" + date
					+ "' where locationName = '" + data.getLocationName() + "'";
			stat.executeUpdate(sql);
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
	}

	/**
	 * T[`L[nXg擾
	 * 
	 * @param key
	 *            T[`L[
	 * @return nXg
	 */
	public List<String> getLocationNameList(String key) {
		List<String> answerList = new ArrayList<String>();
		try {
			String sql = "SELECT locationName FROM geocodinglink "
					+ "WHERE searchQuery = '" + key + "'";
			ResultSet rs = stat.executeQuery(sql);
			int num = 0;
			while (rs.next() && num < returnMax) {
				answerList.add(rs.getString("locationName"));
				num++;
			}
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return answerList;
	}

	/**
	 * T[`L[n擾
	 * 
	 * @param key
	 * @return n񃊃Xg
	 */
	public List<GeoData> getGeodata(String key) {
		List<GeoData> answerList = new ArrayList<GeoData>();
		try {
			String sql = "SELECT geocodingdata.locationName, geocodingdata.latitude, geocodingdata.longitude "
					+ "FROM geocodingrequest, geocodinglink, geocodingdata "
					+ "WHERE geocodingrequest.searchQuery = geocodinglink.searchQuery "
					+ "AND geocodinglink.locationName = geocodingdata.locationName "
					+ "AND geocodingrequest.searchQuery = '" + key + "';";
			ResultSet rs = stat.executeQuery(sql);
			int num = 0;
			while (rs.next() && num < returnMax) {
				GeoData data = new GeoData(rs.getString("locationName"), rs
						.getString("latitude"), rs.getString("longitude"));
				answerList.add(data);
				num++;
			}
		} catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_DB_CONNECT_FAIL,
					e);
			terminate();
		}
		return answerList;
	}

	/**
	 * Geocoding ModuleI
	 */
	private void terminate() {
		System.out.println("Geocoding Module Terminated");// IR\[ɕ\
		closeConnection();
		System.exit(1);
	}

}