package setting;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Properties;

import log.SystemLogger;

/**
 * Geocoding Module̐ݒt@C̏i[IuWFNg
 * 
 * @author yuki
 * @version 1.0
 */
public class GeocodeSettings {

	private final String PROPERTY_FILE_NAME = "geoConfig.prop";

	private int modulePort; // GeocodingModulegp|[gԍ

	private String requestEncode; // o͗p̃GR[h

	private int requestWaitTime; // ÖܓxoxZoT[rX₢킹̑҂

	private int returnMax; // ԂܓxoxyA̐

	private int cacheExpiration; // LbVe[u̍XV

	private int searchKeyExpiration; // NGXge[u̍XV

	private String apiURL; // ÖܓxoxZoT[rXURL

	private String apiEncode; // ÖܓxoxZoT[rX₢킹̃GR[h

	private String logEncode; // Ȍo̓GR[h

	private String logLevel; // VXeȌo͐ݒ

	private String systemLogPath; // VXeȌo̓pX

	private String accessLogPath; // NGXgȌo̓pX

	private String DBURL; // f[^x[XURL

	private String DBUserName; // f[^x[X̃[U

	private String DBPass; // f[^x[X̃pX[h

	/**
	 * ݒt@Cǂݍ
	 * 
	 * @return ݒt@C̓ǂݍ݂ɐtrueAsfalseԂ
	 */
	public boolean readConfigFile() {
		FileInputStream input = null;
		try {
			String configFileAddress = initPath(PROPERTY_FILE_NAME); // RtBOt@ĈpX擾
			input = new FileInputStream(configFileAddress);
			Properties prop = new Properties();
			prop.load(input);

			modulePort = Integer.parseInt(prop.getProperty("modulePort"));
			requestEncode = prop.getProperty("requestEncode");
			requestWaitTime = Integer.parseInt(prop
					.getProperty("requestWaitTime"));
			returnMax = Integer.parseInt(prop.getProperty("returnMax"));
			cacheExpiration = Integer.parseInt(prop
					.getProperty("cacheExpiration"));
			searchKeyExpiration = Integer.parseInt(prop
					.getProperty("searchKeyExpiration"));
			apiURL = prop.getProperty("apiURL");
			apiEncode = prop.getProperty("apiEncode");

			logEncode = prop.getProperty("logEncode");
			logLevel = prop.getProperty("logLevel");
			systemLogPath = initPath(prop.getProperty("systemLogPath"));
			accessLogPath = initPath(prop.getProperty("accessLogPath"));

			DBURL = prop.getProperty("DBURL");
			DBUserName = prop.getProperty("DBUserName");
			DBPass = prop.getProperty("DBPass");

			input.close();
			return true;
		}
		// O
		catch (Exception e) {
			SystemLogger.writer.writeError(SystemLogger.ERROR_CFG_BROKEN, e);
			// inputJĂȑꍇ́A
			if (input != null) {
				try {
					input.close();
				} catch (IOException e1) {
				}
			}
			return false;
		}
	}

	/**
	 * ݒt@CĂȂ`FbN
	 * 
	 * @return ݒt@CȏꍇtrueAĂꍇfalseԂ
	 */
	public boolean checkConfigFile() {
		return notNull() && checkPortNumber(modulePort)
				&& isNaturalNumber(cacheExpiration)
				&& isNaturalNumber(searchKeyExpiration)
				&& isNaturalNumber(returnMax)
				&& isNaturalNumber(requestWaitTime) && logLevelCheck(logLevel)
				&& encodeCheck(logEncode) && encodeCheck(apiEncode)
				&& encodeCheck(requestEncode) && createDir(systemLogPath)
				&& createDir(accessLogPath);
	}

	/**
	 * ϐɒlĂ邩ǂ`FbN
	 * 
	 * @return
	 */
	private boolean notNull() {
		return apiURL != null && logLevel != null && systemLogPath != null
				&& logEncode != null && accessLogPath != null
				&& apiEncode != null && requestEncode != null && DBURL != null
				&& DBUserName != null && DBPass != null;
	}

	/**
	 * |[gԍLȂ̂ł邩ǂ`FbN
	 * 
	 * @param port
	 *            |[gԍ
	 * @return |[gԍL͈͓ȂtrueA͈͊OfalseԂ
	 */
	private boolean checkPortNumber(int port) {
		return port > 0 && port <= 65535;
	}

	/**
	 * lRł邩ǂ`FbN
	 * 
	 * @param num
	 *            l
	 * @return lRȂtrueAłȂȂfalseԂ
	 */
	private boolean isNaturalNumber(int num) {
		return num > 0;
	}

	/**
	 * ̓eonoffǂ`FbN
	 * 
	 * @param input
	 *            
	 * @return ̓e"on""off"łꍇtrueAłȂꍇfalseԂ
	 */
	private boolean logLevelCheck(String input) {
		return input.equals("DEBUG") || input.equals("INFO")
				|| input.equals("WARNING") || input.equals("ERROR");
	}

	/**
	 * pXfBNg쐬
	 * 
	 * @param path
	 *            pX
	 * @return fBNg̍쐬ɐꍇtrueAsꍇfalseԂB
	 *         w肵fBNgɑ݂ꍇ́AtrueԂB
	 */
	public boolean createDir(String path) {
		try {
			if (path == null)
				return false;
			if (pathExist(path) && dirCheck(path))
				return true;
			return (new File(path)).mkdir();
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * pX݂邩ǂ`FbN
	 * 
	 * @param path
	 *            pX
	 * @return w肵pX݂ꍇtrueAȂꍇfalseԂ
	 */
	private boolean pathExist(String path) {
		return (new File(path)).exists();
	}

	/**
	 * pXfBNgǂ`FbN
	 * 
	 * @param path
	 *            pX
	 * @return fBNgłꍇtrueAłȂꍇfalseԂ
	 */
	private boolean dirCheck(String path) {
		return (new File(path)).isDirectory();
	}

	/**
	 * R[hȂ̂ł邩`FbN
	 * 
	 * @param encode
	 *            GR[h
	 * @return ȕR[h̏ꍇtrueAsȏꍇfalseԂ
	 */
	private boolean encodeCheck(String encode) {
		String filename = "./GeoTest.temp";
		File tempFile = new File(filename);
		try {
			FileOutputStream fOut = new FileOutputStream(filename);
			OutputStreamWriter oStream = new OutputStreamWriter(fOut, encode);
			BufferedWriter bWriter = new BufferedWriter(oStream);
			String message = "This is a test";
			bWriter.write(message);
			bWriter.close();
			oStream.close();
			fOut.close();
			return tempFile.delete();
		} catch (Exception e) {
			tempFile.delete();
			return false;
		}
	}

	/**
	 * pXʂiǂłĂׂ悤ɁAϐݒ肳Ăꍇ͌Ăԁj
	 * 
	 * @param path
	 * @return
	 */
	private String initPath(String path) {
		String g_home = System.getenv("G_HOME");// ϐfQgnld擾
		if (g_home != null) {
			path = g_home + "/" + path;// pXʂ
		}
		return path;
	}

	/***************************************************************************
	 * Qb^[\bhS
	 **************************************************************************/

	public int getCacheExpiration() {
		return cacheExpiration;
	}

	public int getModulePort() {
		return modulePort;
	}

	public String getLogLevel() {
		return logLevel;
	}

	public String getLogEncode() {
		return logEncode;
	}

	public String getSystemLogPath() {
		return systemLogPath;
	}

	public String getAccessLogPath() {
		return accessLogPath;
	}

	public String getApiURL() {
		return apiURL;
	}

	public int getReturnMax() {
		return returnMax;
	}

	public String getDBURL() {
		return DBURL;
	}

	public String getApiEncode() {
		return apiEncode;
	}

	public String getRequestEncode() {
		return requestEncode;
	}

	public int getSearchKeyExpiration() {
		return searchKeyExpiration;
	}

	public int getRequestWaitTime() {
		return requestWaitTime;
	}

	public String getDBUserName() {
		return DBUserName;
	}

	public String getDBPass() {
		return DBPass;
	}
}
